/* 
Dreambox RSS reader
Copyright (C) 2004 Bjorn Hijmans (bjorn@hijmans.nl)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <string.h>
#include "rss.h"

extern "C" int plugin_exec( PluginParam *par );

int plugin_exec( PluginParam *par )
{
	rssMain dlg;
	dlg.show();
	int result=dlg.exec();
	dlg.hide();

	return result;
}

eString stringReplace( const eString &toReplace, char from, char to )
{	char buffer[8192];
	strcpy(buffer, (char *)toReplace.c_str());
	char *p = buffer;
	while ( *p != '\0' ) 
	{	if ( *p == from )
			*p = to;
		p++;
	}
	return eString( buffer );
}

eString removeTags(eString in)
{	eString out;
	// Find first <
	int startpos = 0; int length = 0;
	for(unsigned int i = 0; i < in.length(); i++)
	{	length++;
		if(in.mid(i, 1) == "<")
		{	out = out + in.mid(startpos, length - 1);
			// Look for >
			for(unsigned int j = i; j < in.length(); j++)
			{	if(in.mid(j, 1) == ">")
				{	i = j;
					startpos = i + 1;
					length = 0;
					break;
				}
			}
		}
	}
	if(out == "")
	{	out = in;
	}
	return out;
}
			

rssMain::rssMain(): eWindow(1)
{	cmove(ePoint(140, 120));
	cresize(eSize(440, 336));
	setText("Dreambox RSS reader");

	theList = new eListBox<eListBoxEntryText>(this);
	theList->move(ePoint(10, 10));
	theList->resize(eSize(clientrect.width() - 20, clientrect.height() - 20));
	theList->loadDeco();
	theList->setColumns(1);

	setFocus(theList);

	if (!theConfigParser.parse("/var/tuxbox/config/feeds.xml"))
		theConfigParser.parse(CONFIGDIR"/feeds.xml");

	theRssFeed = new rssFeed();
	theRssFeed->hide();

	inDownloadFlag = 0;

	printFeeds();	

	CONNECT(theList->selected, rssMain::selectedItem);
}

void rssMain::printFeeds()
{	std::list<ConfigItem>::iterator i;
	for(i = theConfigParser.configItems.begin(); i != theConfigParser.configItems.end(); ++i)
	{	new eListBoxEntryText(theList, i->name.c_str(), (void *) &(i->id));
	}
}

void rssMain::selectedItem(eListBoxEntryText *item)
{	if(item && !inDownloadFlag)
	{	inDownloadFlag = 1;

		int * iKey = (int *) (item->getKey());

		eString url;
		eString name;

		std::list<ConfigItem>::iterator i;
		for(i = theConfigParser.configItems.begin(); i != theConfigParser.configItems.end(); ++i)
		{	if(i->id == *iKey)
			{	url = i->url;
				name = i->name;
			}
		}

		CONNECT(theFetcher.downloadDone, rssMain::downloadDone);		
		downloadDoneFlag = 0;
		currentName = name;
		theFetcher.fetch(url);
	}
}

void rssMain::downloadDone(int err)
{	if(!downloadDoneFlag)
	{	downloadDoneFlag = 1;
		this->hide();
		theRssFeed->printRSSFeed(currentName);
		theRssFeed->show();
		theRssFeed->exec();
		theRssFeed->hide();
		this->show();
		setFocus(theList);
		inDownloadFlag = 0;
	}
}

rssFeed::rssFeed(): eWindow(1)
{	
	//720x576
	//560x376

	move(ePoint(80, 100));
	resize(eSize(560, 376));
	setText("Feed");

	theList = new eListBox<eListBoxEntryText>(this);
	theList->move(ePoint(10, 10));
	theList->resize(eSize(clientrect.width() - 20, clientrect.height() - 20));
	theList->loadDeco();
	theList->setColumns(1);

	setFocus(theList);

	CONNECT(theList->selected, rssFeed::selectedItem);
}

void rssFeed::printRSSFeed(eString title)
{	eString dest = "/var/tmp/rss.tmp";
	theRSSParser.parse(dest);
	
	setFocus(theList);
	setText(title);
	
	theList->beginAtomic();
	theList->clearList();
	
	if(theList)
	{	// Iterate through list
		std::list<NewsItem>::iterator i;
		for(i = theRSSParser.newsItems.begin(); i != theRSSParser.newsItems.end(); ++i)
		{	new eListBoxEntryText(theList, i->title.c_str(), (void *) (i->id));
		}
	}
	theList->endAtomic();

}

void rssFeed::selectedItem(eListBoxEntryText *item)
{	if(item)
	{	int iKey = (int) (item->getKey());

		eString description;
		eString title;

		std::list<NewsItem>::iterator i;
		for(i = theRSSParser.newsItems.begin(); i != theRSSParser.newsItems.end(); ++i)
		{	if(i->id == iKey)
			{	description = i->description;
				title = i->title;
			}
		}

		//eMessageBox msg(description.c_str(), title.c_str(), eMessageBox::btOK);
		//msg.show();     msg.exec();     msg.hide();
		
		rssDetail dlg(title.c_str(), description.c_str());
		dlg.show();
		dlg.exec();
		dlg.hide();
	}
	else
	{	reject();
	}
}

rssMain::~rssMain()
{
	// we have to do almost nothing here. all widgets are automatically removed
	// since they are children of the main dialog. the eWidget-destructor will do to this.
}

void Fetcher::fetch(eString url)
{	tempPath = "/var/tmp/rss.tmp";
	int error = 0;
	connectionP = eHTTPConnection::doRequest(url.c_str(), eApp, &error);
	
	if(!connectionP || error)
	{	eMessageBox msg("Error downloading " + url + "(" + eString().sprintf("%d", error) + ")", _("Details"), eMessageBox::btOK);
		msg.show();     msg.exec();     msg.hide();
	}
	else
	{	CONNECT(connectionP->transferDone, Fetcher::transferDone);
		CONNECT(connectionP->createDataSource, Fetcher::createDownloadSink);
		connectionP->local_header["User-Agent"] = "RSS";
		connectionP->start();
	}
}

void Fetcher::transferDone(int err)
{	
	if(!err)
	{	connectionP = NULL;

		// Tell caller download is ready
		/*emit*/ downloadDone(err);
	}
	else
	{	eString sMsg = "Error " + eString().sprintf("%d", err);
		eMessageBox msg(sMsg, _("User Abort"), eMessageBox::iconWarning|eMessageBox::btOK);
		msg.show();     msg.exec();     msg.hide();
	}
}

eHTTPDataSource * Fetcher::createDownloadSink(eHTTPConnection *conn)
{	dataSinkP = new eHTTPDownload(connectionP, (char *)tempPath.c_str());

	return(dataSinkP);
}

void ConfigParser::save(ConfigItem i)
{	configItems.push_back(i);
}

// Text ersetzen
void replace(char * pointer, char * alt, char * neu){
char *  dummy;
char *  start;
char *  ziel;
int     laenge;

    if (strstr(pointer, alt)==NULL) return;
    dummy=pointer;
    ziel=pointer;
    while (dummy!=NULL){
        start=dummy;
        dummy=strstr(dummy, alt);
        if (dummy!=NULL){
            laenge=dummy-start;
            if (laenge>=0){
                memcpy(ziel, start, laenge);
                ziel+=laenge;
                strcpy(ziel, neu);
                ziel+=strlen(neu);
            }
            dummy+=strlen(alt);
        }
    }
    strcpy(ziel, start);
}


// HTML in Textdateien wandeln
void replace_sonderzeichen(char * pointer){

// Sonderzeichen ersetzen
    replace(pointer, "&nbsp;", " ");
    replace(pointer, "&quot;", "\"");
    replace(pointer, "&apos;", "'");
    replace(pointer, "&gt;", ">");
    replace(pointer, "&lt;", "<");
    replace(pointer, "&amp;", "&");
// Sonderzeichen ersetzen
    replace(pointer, "&auml;", "ä");
    replace(pointer, "&ouml;", "ö");
    replace(pointer, "&uuml;", "ü");
    replace(pointer, "&Auml;", "Ä");
    replace(pointer, "&Ouml;", "Ö");
    replace(pointer, "&Uuml;", "Ü");
    replace(pointer, "&szlig;", "ß");
    replace(pointer, "&copy;", "\169");
    replace(pointer, "&shy;", "\173");
    replace(pointer, "&Agrave;", "\192");
    replace(pointer, "&Aacute;", "\193");
    replace(pointer, "&Acirc;", "\194");
    replace(pointer, "&Atilde;", "\195");
    replace(pointer, "&Aring;", "\197");
    replace(pointer, "&AElig;", "\198");
    replace(pointer, "&Ccedil;", "\199");
    replace(pointer, "&Egrave;", "\200");
    replace(pointer, "&Eacute;", "\201");
    replace(pointer, "&Ecirc;", "\202");
    replace(pointer, "&Euml;", "\203");
    replace(pointer, "&Igrave;", "\204");
    replace(pointer, "&Iacute;", "\205");
    replace(pointer, "&Icirc;", "\206");
    replace(pointer, "&Iuml;", "\207");
    replace(pointer, "&ETH;", "\208");
    replace(pointer, "&Ntilde;", "\209");
    replace(pointer, "&Ograve;", "\210");
    replace(pointer, "&Oacute;", "\211");
    replace(pointer, "&Ocirc;", "\212");
    replace(pointer, "&Otilde;", "\213");
    replace(pointer, "&Oslash;", "\216");
    replace(pointer, "&Ugrave;", "\217");
    replace(pointer, "&Uacute;", "\218");
    replace(pointer, "&Ucirc;", "\219");
    replace(pointer, "&Yacute;", "\221");
    replace(pointer, "&THORN;", "\222");
    replace(pointer, "&agrave;", "\224");
    replace(pointer, "&aacute;", "\225");
    replace(pointer, "&acirc;", "\226");
    replace(pointer, "&atilde;", "\227");
    replace(pointer, "&aring;", "\229");
    replace(pointer, "&aelig;", "\230");
    replace(pointer, "&ccedil;", "\231");
    replace(pointer, "&egrave;", "\232");
    replace(pointer, "&eacute;", "\233");
    replace(pointer, "&ecirc;", "\234");
    replace(pointer, "&euml;", "\235");
    replace(pointer, "&igrave;", "\236");
    replace(pointer, "&iacute;", "\237");
    replace(pointer, "&icirc;", "\238");
    replace(pointer, "&iuml;", "\239");
    replace(pointer, "&eth;", "\240");
    replace(pointer, "&ntilde;", "\241");
    replace(pointer, "&ograve;", "\242");
    replace(pointer, "&oacute;", "\243");
    replace(pointer, "&ocirc;", "\244");
    replace(pointer, "&otilde;", "\245");
    replace(pointer, "&oslash;", "\248");
    replace(pointer, "&ugrave;", "\249");
    replace(pointer, "&uacute;", "\250");
    replace(pointer, "&ucirc;", "\251");
    replace(pointer, "&yacute;", "\253");
    replace(pointer, "&thorn;", "\254");
    replace(pointer, "&yuml;", "\255");
    replace(pointer, "&#169;", "©");
    replace(pointer, "&middot;", "·");
    replace(pointer, "<br>;", "\10\13");
}
		// 8859-x to ucs-16 coding tables. taken from www.unicode.org/Public/MAPPINGS/ISO8859/

static unsigned long c88592[96]={
0x00A0, 0x0104, 0x02D8, 0x0141, 0x00A4, 0x013D, 0x015A, 0x00A7, 0x00A8, 0x0160, 0x015E, 0x0164, 0x0179, 0x00AD, 0x017D, 0x017B,
0x00B0, 0x0105, 0x02DB, 0x0142, 0x00B4, 0x013E, 0x015B, 0x02C7, 0x00B8, 0x0161, 0x015F, 0x0165, 0x017A, 0x02DD, 0x017E, 0x017C,
0x0154, 0x00C1, 0x00C2, 0x0102, 0x00C4, 0x0139, 0x0106, 0x00C7, 0x010C, 0x00C9, 0x0118, 0x00CB, 0x011A, 0x00CD, 0x00CE, 0x010E,
0x0110, 0x0143, 0x0147, 0x00D3, 0x00D4, 0x0150, 0x00D6, 0x00D7, 0x0158, 0x016E, 0x00DA, 0x0170, 0x00DC, 0x00DD, 0x0162, 0x00DF,
0x0155, 0x00E1, 0x00E2, 0x0103, 0x00E4, 0x013A, 0x0107, 0x00E7, 0x010D, 0x00E9, 0x0119, 0x00EB, 0x011B, 0x00ED, 0x00EE, 0x010F,
0x0111, 0x0144, 0x0148, 0x00F3, 0x00F4, 0x0151, 0x00F6, 0x00F7, 0x0159, 0x016F, 0x00FA, 0x0171, 0x00FC, 0x00FD, 0x0163, 0x02D9};
	
static unsigned long c88593[96]={
0x00A0, 0x0126, 0x02D8, 0x00A3, 0x00A4, 0x0000, 0x0124, 0x00A7, 0x00A8, 0x0130, 0x015E, 0x011E, 0x0134, 0x00AD, 0x0000, 0x017B,
0x00B0, 0x0127, 0x00B2, 0x00B3, 0x00B4, 0x00B5, 0x0125, 0x00B7, 0x00B8, 0x0131, 0x015F, 0x011F, 0x0135, 0x00BD, 0x0000, 0x017C,
0x00C0, 0x00C1, 0x00C2, 0x0000, 0x00C4, 0x010A, 0x0108, 0x00C7, 0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x0000, 0x00D1, 0x00D2, 0x00D3, 0x00D4, 0x0120, 0x00D6, 0x00D7, 0x011C, 0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x016C, 0x015C, 0x00DF,
0x00E0, 0x00E1, 0x00E2, 0x0000, 0x00E4, 0x010B, 0x0109, 0x00E7, 0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF,
0x0000, 0x00F1, 0x00F2, 0x00F3, 0x00F4, 0x0121, 0x00F6, 0x00F7, 0x011D, 0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x016D, 0x015D, 0x02D9};

static unsigned long c88594[96]={
0x00A0, 0x0104, 0x0138, 0x0156, 0x00A4, 0x0128, 0x013B, 0x00A7, 0x00A8, 0x0160, 0x0112, 0x0122, 0x0166, 0x00AD, 0x017D, 0x00AF,
0x00B0, 0x0105, 0x02DB, 0x0157, 0x00B4, 0x0129, 0x013C, 0x02C7, 0x00B8, 0x0161, 0x0113, 0x0123, 0x0167, 0x014A, 0x017E, 0x014B,
0x0100, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x012E, 0x010C, 0x00C9, 0x0118, 0x00CB, 0x0116, 0x00CD, 0x00CE, 0x012A,
0x0110, 0x0145, 0x014C, 0x0136, 0x00D4, 0x00D5, 0x00D6, 0x00D7, 0x00D8, 0x0172, 0x00DA, 0x00DB, 0x00DC, 0x0168, 0x016A, 0x00DF,
0x0101, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x012F, 0x010D, 0x00E9, 0x0119, 0x00EB, 0x0117, 0x00ED, 0x00EE, 0x012B,
0x0111, 0x0146, 0x014D, 0x0137, 0x00F4, 0x00F5, 0x00F6, 0x00F7, 0x00F8, 0x0173, 0x00FA, 0x00FB, 0x00FC, 0x0169, 0x016B, 0x02D9};

static unsigned long c88595[96]={
0x00A0, 0x0401, 0x0402, 0x0403, 0x0404, 0x0405, 0x0406, 0x0407, 0x0408, 0x0409, 0x040A, 0x040B, 0x040C, 0x00AD, 0x040E, 0x040F,
0x0410, 0x0411, 0x0412, 0x0413, 0x0414, 0x0415, 0x0416, 0x0417, 0x0418, 0x0419, 0x041A, 0x041B, 0x041C, 0x041D, 0x041E, 0x041F,
0x0420, 0x0421, 0x0422, 0x0423, 0x0424, 0x0425, 0x0426, 0x0427, 0x0428, 0x0429, 0x042A, 0x042B, 0x042C, 0x042D, 0x042E, 0x042F,
0x0430, 0x0431, 0x0432, 0x0433, 0x0434, 0x0435, 0x0436, 0x0437, 0x0438, 0x0439, 0x043A, 0x043B, 0x043C, 0x043D, 0x043E, 0x043F,
0x0440, 0x0441, 0x0442, 0x0443, 0x0444, 0x0445, 0x0446, 0x0447, 0x0448, 0x0449, 0x044A, 0x044B, 0x044C, 0x044D, 0x044E, 0x044F,
0x2116, 0x0451, 0x0452, 0x0453, 0x0454, 0x0455, 0x0456, 0x0457, 0x0458, 0x0459, 0x045A, 0x045B, 0x045C, 0x00A7, 0x045E, 0x045F};
	
static unsigned long c88596[96]={
0x00A0, 0x0000, 0x0000, 0x0000, 0x00A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x060C, 0x00AD, 0x0000, 0x0000,
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x061B, 0x0000, 0x0000, 0x0000, 0x061F,
0x0000, 0x0621, 0x0622, 0x0623, 0x0624, 0x0625, 0x0626, 0x0627, 0x0628, 0x0629, 0x062A, 0x062B, 0x062C, 0x062D, 0x062E, 0x062F,
0x0630, 0x0631, 0x0632, 0x0633, 0x0634, 0x0635, 0x0636, 0x0637, 0x0638, 0x0639, 0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
0x0640, 0x0641, 0x0642, 0x0643, 0x0644, 0x0645, 0x0646, 0x0647, 0x0648, 0x0649, 0x064A, 0x064B, 0x064C, 0x064D, 0x064E, 0x064F,
0x0650, 0x0651, 0x0652, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000};
	
static unsigned long c88597[96]={
0x00A0, 0x2018, 0x2019, 0x00A3, 0x20AC, 0x20AF, 0x00A6, 0x00A7, 0x00A8, 0x00A9, 0x037A, 0x00AB, 0x00AC, 0x00AD, 0x0000, 0x2015,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x0384, 0x0385, 0x0386, 0x00B7, 0x0388, 0x0389, 0x038A, 0x00BB, 0x038C, 0x00BD, 0x038E, 0x038F,
0x0390, 0x0391, 0x0392, 0x0393, 0x0394, 0x0395, 0x0396, 0x0397, 0x0398, 0x0399, 0x039A, 0x039B, 0x039C, 0x039D, 0x039E, 0x039F,
0x03A0, 0x03A1, 0x0000, 0x03A3, 0x03A4, 0x03A5, 0x03A6, 0x03A7, 0x03A8, 0x03A9, 0x03AA, 0x03AB, 0x03AC, 0x03AD, 0x03AE, 0x03AF,
0x03B0, 0x03B1, 0x03B2, 0x03B3, 0x03B4, 0x03B5, 0x03B6, 0x03B7, 0x03B8, 0x03B9, 0x03BA, 0x03BB, 0x03BC, 0x03BD, 0x03BE, 0x03BF,
0x03C0, 0x03C1, 0x03C2, 0x03C3, 0x03C4, 0x03C5, 0x03C6, 0x03C7, 0x03C8, 0x03C9, 0x03CA, 0x03CB, 0x03CC, 0x03CD, 0x03CE, 0x0000};

static unsigned long c88598[96]={
0x00A0, 0x0000, 0x00A2, 0x00A3, 0x00A4, 0x00A5, 0x00A6, 0x00A7, 0x00A8, 0x00A9, 0x00D7, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x00AF,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x00B4, 0x00B5, 0x00B6, 0x00B7, 0x00B8, 0x00B9, 0x00F7, 0x00BB, 0x00BC, 0x00BD, 0x00BE, 0x0000,
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x2017,
0x05D0, 0x05D1, 0x05D2, 0x05D3, 0x05D4, 0x05D5, 0x05D6, 0x05D7, 0x05D8, 0x05D9, 0x05DA, 0x05DB, 0x05DC, 0x05DD, 0x05DE, 0x05DF,
0x05E0, 0x05E1, 0x05E2, 0x05E3, 0x05E4, 0x05E5, 0x05E6, 0x05E7, 0x05E8, 0x05E9, 0x05EA, 0x0000, 0x0000, 0x200E, 0x200F, 0x0000};

static unsigned long c88599[96]={
0x00A0, 0x00A1, 0x00A2, 0x00A3, 0x00A4, 0x00A5, 0x00A6, 0x00A7, 0x00A8, 0x00A9, 0x00AA, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x00AF,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x00B4, 0x00B5, 0x00B6, 0x00B7, 0x00B8, 0x00B9, 0x00BA, 0x00BB, 0x00BC, 0x00BD, 0x00BE, 0x00BF,
0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x00C7, 0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x011E, 0x00D1, 0x00D2, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x00D7, 0x00D8, 0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x0130, 0x015E, 0x00DF,
0x00E0, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x00E7, 0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF,
0x011F, 0x00F1, 0x00F2, 0x00F3, 0x00F4, 0x00F5, 0x00F6, 0x00F7, 0x00F8, 0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x0131, 0x015F, 0x00FF};

static unsigned long c885910[96]={
0x00A0, 0x0104, 0x0112, 0x0122, 0x012A, 0x0128, 0x0136, 0x00A7, 0x013B, 0x0110, 0x0160, 0x0166, 0x017D, 0x00AD, 0x016A, 0x014A,
0x00B0, 0x0105, 0x0113, 0x0123, 0x012B, 0x0129, 0x0137, 0x00B7, 0x013C, 0x0111, 0x0161, 0x0167, 0x017E, 0x2015, 0x016B, 0x014B,
0x0100, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x012E, 0x010C, 0x00C9, 0x0118, 0x00CB, 0x0116, 0x00CD, 0x00CE, 0x00CF,
0x00D0, 0x0145, 0x014C, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x0168, 0x00D8, 0x0172, 0x00DA, 0x00DB, 0x00DC, 0x00DD, 0x00DE, 0x00DF,
0x0101, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x012F, 0x010D, 0x00E9, 0x0119, 0x00EB, 0x0117, 0x00ED, 0x00EE, 0x00EF,
0x00F0, 0x0146, 0x014D, 0x00F3, 0x00F4, 0x00F5, 0x00F6, 0x0169, 0x00F8, 0x0173, 0x00FA, 0x00FB, 0x00FC, 0x00FD, 0x00FE, 0x0138};
	
static unsigned long c885911[96]={
0x00A0, 0x0E01, 0x0E02, 0x0E03, 0x0E04, 0x0E05, 0x0E06, 0x0E07, 0x0E08, 0x0E09, 0x0E0A, 0x0E0B, 0x0E0C, 0x0E0D, 0x0E0E, 0x0E0F,
0x0E10, 0x0E11, 0x0E12, 0x0E13, 0x0E14, 0x0E15, 0x0E16, 0x0E17, 0x0E18, 0x0E19, 0x0E1A, 0x0E1B, 0x0E1C, 0x0E1D, 0x0E1E, 0x0E1F,
0x0E20, 0x0E21, 0x0E22, 0x0E23, 0x0E24, 0x0E25, 0x0E26, 0x0E27, 0x0E28, 0x0E29, 0x0E2A, 0x0E2B, 0x0E2C, 0x0E2D, 0x0E2E, 0x0E2F,
0x0E30, 0x0E31, 0x0E32, 0x0E33, 0x0E34, 0x0E35, 0x0E36, 0x0E37, 0x0E38, 0x0E39, 0x0E3A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0E3F,
0x0E40, 0x0E41, 0x0E42, 0x0E43, 0x0E44, 0x0E45, 0x0E46, 0x0E47, 0x0E48, 0x0E49, 0x0E4A, 0x0E4B, 0x0E4C, 0x0E4D, 0x0E4E, 0x0E4F,
0x0E50, 0x0E51, 0x0E52, 0x0E53, 0x0E54, 0x0E55, 0x0E56, 0x0E57, 0x0E58, 0x0E59, 0x0E5A, 0x0E5B, 0x0000, 0x0000, 0x0000, 0x0000};

static unsigned long c885913[96]={
0x00A0, 0x201D, 0x00A2, 0x00A3, 0x00A4, 0x201E, 0x00A6, 0x00A7, 0x00D8, 0x00A9, 0x0156, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x00C6,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x201C, 0x00B5, 0x00B6, 0x00B7, 0x00F8, 0x00B9, 0x0157, 0x00BB, 0x00BC, 0x00BD, 0x00BE, 0x00E6,
0x0104, 0x012E, 0x0100, 0x0106, 0x00C4, 0x00C5, 0x0118, 0x0112, 0x010C, 0x00C9, 0x0179, 0x0116, 0x0122, 0x0136, 0x012A, 0x013B,
0x0160, 0x0143, 0x0145, 0x00D3, 0x014C, 0x00D5, 0x00D6, 0x00D7, 0x0172, 0x0141, 0x015A, 0x016A, 0x00DC, 0x017B, 0x017D, 0x00DF,
0x0105, 0x012F, 0x0101, 0x0107, 0x00E4, 0x00E5, 0x0119, 0x0113, 0x010D, 0x00E9, 0x017A, 0x0117, 0x0123, 0x0137, 0x012B, 0x013C,
0x0161, 0x0144, 0x0146, 0x00F3, 0x014D, 0x00F5, 0x00F6, 0x00F7, 0x0173, 0x0142, 0x015B, 0x016B, 0x00FC, 0x017C, 0x017E, 0x2019};

static unsigned long c885914[96]={
0x00A0, 0x1E02, 0x1E03, 0x00A3, 0x010A, 0x010B, 0x1E0A, 0x00A7, 0x1E80, 0x00A9, 0x1E82, 0x1E0B, 0x1EF2, 0x00AD, 0x00AE, 0x0178,
0x1E1E, 0x1E1F, 0x0120, 0x0121, 0x1E40, 0x1E41, 0x00B6, 0x1E56, 0x1E81, 0x1E57, 0x1E83, 0x1E60, 0x1EF3, 0x1E84, 0x1E85, 0x1E61,
0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x00C7, 0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x0174, 0x00D1, 0x00D2, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x1E6A, 0x00D8, 0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x00DD, 0x0176, 0x00DF,
0x00E0, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x00E7, 0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF,
0x0175, 0x00F1, 0x00F2, 0x00F3, 0x00F4, 0x00F5, 0x00F6, 0x1E6B, 0x00F8, 0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x00FD, 0x0177, 0x00FF};

static unsigned long c885915[96]={
0x00A0, 0x00A1, 0x00A2, 0x00A3, 0x20AC, 0x00A5, 0x0160, 0x00A7, 0x0161, 0x00A9, 0x00AA, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x00AF,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x017D, 0x00B5, 0x00B6, 0x00B7, 0x017E, 0x00B9, 0x00BA, 0x00BB, 0x0152, 0x0153, 0x0178, 0x00BF,
0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x00C7, 0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x00D0, 0x00D1, 0x00D2, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x00D7, 0x00D8, 0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x00DD, 0x00DE, 0x00DF,
0x00E0, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x00E7, 0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF,
0x00F0, 0x00F1, 0x00F2, 0x00F3, 0x00F4, 0x00F5, 0x00F6, 0x00F7, 0x00F8, 0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x00FD, 0x00FE, 0x00FF};

static unsigned long c885916[96]={
0x00A0, 0x0104, 0x0105, 0x0141, 0x20AC, 0x201E, 0x0160, 0x00A7, 0x0161, 0x00A9, 0x0218, 0x00AB, 0x0179, 0x00AD, 0x017A, 0x017B,
0x00B0, 0x00B1, 0x010C, 0x0142, 0x017D, 0x201D, 0x00B6, 0x00B7, 0x017E, 0x010D, 0x0219, 0x00BB, 0x0152, 0x0153, 0x0178, 0x017C,
0x00C0, 0x00C1, 0x00C2, 0x0102, 0x00C4, 0x0106, 0x00C6, 0x00C7, 0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x0110, 0x0143, 0x00D2, 0x00D3, 0x00D4, 0x0150, 0x00D6, 0x015A, 0x0170, 0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x0118, 0x021A, 0x00DF,
0x00E0, 0x00E1, 0x00E2, 0x0103, 0x00E4, 0x0107, 0x00E6, 0x00E7, 0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF,
0x0111, 0x0144, 0x00F2, 0x00F3, 0x00F4, 0x0151, 0x00F6, 0x015B, 0x0171, 0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x0119, 0x021B, 0x00FF};

static inline unsigned int recode(unsigned char d, int cp)
{
	if (d < 0xA0)
		return d;
	switch (cp)
	{
	case 0:		// Latin1 <-> unicode mapping
	case 1:		// 8859-1 <-> unicode mapping
		return d;
	case 2:		// 8859-2 -> unicode mapping
		return c88592[d-0xA0];
	case 3:		// 8859-3 -> unicode mapping
		return c88593[d-0xA0];
	case 4:		// 8859-2 -> unicode mapping
		return c88594[d-0xA0];
	case 5:		// 8859-5 -> unicode mapping
		return c88595[d-0xA0];
	case 6:		// 8859-6 -> unicode mapping
		return c88596[d-0xA0];
	case 7:		// 8859-7 -> unicode mapping
		return c88597[d-0xA0];
	case 8:		// 8859-8 -> unicode mapping
		return c88598[d-0xA0];
	case 9:		// 8859-9 -> unicode mapping
		return c88599[d-0xA0];
	case 10:// 8859-10 -> unicode mapping
		return c885910[d-0xA0];
	case 11:// 8859-11 -> unicode mapping
		return c885911[d-0xA0];
/*	case 12:// 8859-12 -> unicode mapping  // reserved for indian use..
		return c885912[d-0xA0];*/
	case 13:// 8859-13 -> unicode mapping
		return c885913[d-0xA0];
	case 14:// 8859-14 -> unicode mapping
		return c885914[d-0xA0];
	case 15:// 8859-15 -> unicode mapping
		return c885915[d-0xA0];
	case 16:// 8859-16 -> unicode mapping
		return c885916[d-0xA0];
	default:
		return d;
	}
}

int convert2UTF8(char *src, char *dest, int len, int table)
{
	if (!len)
		return 0;

	int i=0, t=0;

	while (i < len)
	{
		unsigned long code=0;

		code=recode(src[i++], table);
		if (!code)
			continue;
				// Unicode->UTF8 encoding
		if (code < 0x80) // identity ascii <-> utf8 mapping
			dest[t++]=char(code);
		else if (code < 0x800) // two byte mapping
		{
			dest[t++]=(code>>6)|0xC0;
			dest[t++]=(code&0x3F)|0x80;
		} else if (code < 0x10000) // three bytes mapping
		{
			dest[t++]=(code>>12)|0xE0;
			dest[t++]=((code>>6)&0x3F)|0x80;
			dest[t++]=(code&0x3F)|0x80;
		} else
		{
			dest[t++]=(code>>18)|0xF0;
			dest[t++]=((code>>12)&0x3F)|0x80;
			dest[t++]=((code>>6)&0x3F)|0x80;
			dest[t++]=(code&0x3F)|0x80;
		}
		if (t+4 > 2*len)
		{
			eDebug("convertDVBUTF8 buffer to small.. break now");
			break;
		}
	}
	return t;
}

int ConfigParser::parse(eString file)
{	XMLTreeParser * parser;
	FILE *in = fopen(file.c_str(), "rt");

        if(in) 
	{	char buf[2048];
 		char encoding[256];

		int done;
		sprintf(encoding,"ISO-8859-1");
		unsigned int len=fread(buf, 1, sizeof(buf), in);
		char * pointer=strstr(buf, "encoding=");
		if (pointer!=NULL) pointer=strstr(pointer, "\"");
		if (pointer!=NULL){
		    pointer++;
		    int i=0;
		    while ((*pointer)!='\"')
		    {
			encoding[i++]=*pointer++;
		    }
	    	    encoding[i]=0;	
		}
		parser = new XMLTreeParser(encoding);
		do 
		{	done = ( len < sizeof(buf) );
			if ( ! parser->Parse( buf, len, done ) ) 
			{	eMessageBox msg(_("Configfile parse error"), _("User Abort"), eMessageBox::iconWarning|eMessageBox::btOK);
		                msg.show();     msg.exec();     msg.hide();
				delete parser;
				parser = NULL;
				return 0;
			}
			len=fread(buf, 1, sizeof(buf), in);
		} 
		while (!done);

                fclose(in);

		XMLTreeNode * root = parser->RootNode();
		if(!root)
		{	eMessageBox msg(_("Configfile parse error"), _("User Abort"), eMessageBox::iconWarning|eMessageBox::btOK);
		        msg.show();     msg.exec();     msg.hide();	
			return 0;
		}
		if ( strcmp( root->GetType(), "feeds") ) 
		{	eMessageBox msg(_("Invalid configfile"), _("User Abort"), eMessageBox::iconWarning|eMessageBox::btOK);
		        msg.show();     msg.exec();     msg.hide();
	                return 0;
	        }
	
		ConfigItem thisItem;
		int idcnt = 0;

		for(XMLTreeNode * node = root->GetChild(); node; node = node->GetNext())
		{	if(!strcmp(node->GetType(), "feed"))
			{	for(XMLTreeNode * i = node->GetChild(); i; i = i->GetNext())
				{	if(!strcmp(i->GetType(), "name"))
					{	thisItem.name = i->GetData();
						replace_sonderzeichen((char *)thisItem.name.c_str());
					}
					if(!strcmp(i->GetType(), "url"))
					{	thisItem.url = i->GetData();
					}
				}
				thisItem.id = idcnt++;
				save(thisItem);
			}
		}


		delete parser;
		return 1;
	}
	else
	{	eMessageBox msg(_("Config file not found"), _("User Abort"), eMessageBox::iconWarning|eMessageBox::btOK);
		msg.show();     msg.exec();     msg.hide();
		return 0;
	}
}

rssDetail::rssDetail(const char *title, const char *desc) : eWindow(0)
{
	setText(title);

	move(ePoint(80, 123));
	resize(eSize(560, 353));	

	scrollbar = new eProgress(this);
        scrollbar->setStart(0);
        scrollbar->setPerc(100);
        scrollbar->setDirection(1);
        scrollbar->move(ePoint(clientrect.width() - 20, 0));
        scrollbar->resize(eSize(20, clientrect.height() - 30));
        
	descrWidget = new eLabel(this);
	descrWidget->move(ePoint(20, 20));
	descrWidget->resize(eSize(clientrect.width() - 50, clientrect.height() - 50 - 5));
	
	descrLabel = new eLabel(descrWidget);
	descrLabel->setText(desc);
	descrLabel->move(ePoint(0, 0));
	descrLabel->resize(eSize(10, 10));
	descrLabel->setFlags(RS_WRAP);

	float lineheight=fontRenderClass::getInstance()->getLineHeight( descrLabel->getFont() );
	int lines = descrWidget->getSize().height() / (int)lineheight;
	//pageHeight = (int)(lines * lineheight);
	int newheight = lines * (int)lineheight + (int)(round(lineheight + 0.5) - (int)lineheight);
	descrWidget->resize( eSize( descrWidget->getSize().width(), newheight + (int)lineheight/6 ));
	descrLabel->resize( 
		eSize(
			descrWidget->getSize().width(), 
			descrWidget->getSize().height() * 4
		 )
	);

	const eRect &cr = getClientRect();
	eButton * ok = new eButton(this);
	ok->setText("OK");
	ok->move(ePoint((cr.width() - 78)/2, cr.height() - 30));
	ok->resize(eSize(78, 20));
	ok->setShortcut("green");
	ok->setShortcutPixmap("green");

        CONNECT(ok->selected, eWidget::accept);

	updateScrollbar();
}

void rssDetail::updateScrollbar()
{
	total = descrWidget->getSize().height();
	int pages=1;
	while( total < descrLabel->getExtend().height() ) {
		total += descrWidget->getSize().height();
		pages++;
	}

	int start =- ( descrLabel->getPosition().y() * 100 ) /total;
	int vis = descrWidget->getSize().height() *100/total;

	scrollbar->setParams(start, vis);
	scrollbar->show();
	if (pages == 1)
		total = 0;
}

int rssDetail::eventHandler(const eWidgetEvent &event)
{
	switch (event.type) {
        	case eWidgetEvent::evtKey:
			if ( (event.key)->flags == KEY_STATE_DOWN ) {
				switch ( (event.key)->code  ) {
					case RC_UP:
						handleUp();
						break;
					case RC_DN:
						handleDown();
						break;
				}
			}
			break;
		default:
			break;
	}

	return eWindow::eventHandler(event);
}

void rssDetail::handleDown( void )
{
	ePoint curPos = descrLabel->getPosition();
	if ( 
		total &&
		(total - descrWidget->getSize().height() ) >= abs( curPos.y() - descrWidget->getSize().height() ) 
	) {
		descrLabel->move( 
			ePoint( 
				curPos.x(), 
				curPos.y() - descrWidget->getSize().height() 
			)
		);
		updateScrollbar();
	}
}

void rssDetail::handleUp( void )
{
	ePoint curPos = descrLabel->getPosition();
	if ( 
		total && 
		( curPos.y() < 0 )
	) {
		descrLabel->move( 
			ePoint( 
				curPos.x(), 
				curPos.y() + descrWidget->getSize().height() 
			) 
		);
		updateScrollbar();
	}
}

void rssDetail::redrawWidget(gPainter *d, const eRect &area)
{	eWindow::redrawWidget( d, area );
}


void RSSParser::save(NewsItem i)
{	newsItems.push_back(i);
}

#define buf_len	2048

void RSSParser::parse(eString file)
{	XMLTreeParser * parser;
	FILE *in = fopen(file.c_str(), "rt");
	newsItems.clear();

        if(in) 
	{	char 	buf[buf_len];
		char	buf1[2*buf_len];
 		int 	table, len1;

		int done;
		unsigned int len=fread(buf, 1, sizeof(buf), in);
		char * pointer=strstr(buf, "8859-");
		table=0;
		if (pointer!=NULL){
		    sscanf(&pointer[5],"%d",&table);		
 		}
		if (pointer==NULL){
		    pointer=strstr(buf, "-1250");
		    if (pointer!=NULL){
			table=2;
		    }
		}
		if (pointer==NULL){
		    pointer=strstr(buf, "-1252");
		    if (pointer!=NULL){
			table=1;
		    }
		}
		if (pointer==NULL){
		    pointer=strstr(buf, "-1253");
		    if (pointer!=NULL){
			table=7;
		    }
		}
		if (pointer==NULL){
		    pointer=strstr(buf, "-1254");
		    if (pointer!=NULL){
			table=9;
		    }
		}
		pointer=strstr(buf, "UTF-8");
		if (pointer==NULL) pointer=strstr(buf, "utf-8");
		if ((pointer==NULL)&&(table>0)&&(table!=12)&&(table<17)){
		    len1=convert2UTF8(buf, buf1, len, table);
		    parser = new XMLTreeParser("UTF-8");
		}
		else if ((pointer==NULL)&&(table==0)){
		    memcpy(buf1, buf, len);
		    len1=len;
		    parser = new XMLTreeParser("8859-1");
		}
		else{
		    memcpy(buf1, buf, len);
		    len1=len;
		    parser = new XMLTreeParser("UTF-8");
		}
		do 
		{	done = ( len < sizeof(buf) );
			if ( ! parser->Parse( buf1, len1, done ) ) 
			{	eMessageBox msg(_("XML parse error (general xml)"), _("Error"), eMessageBox::iconWarning|eMessageBox::btOK);
		                msg.show();     msg.exec();     msg.hide();
				delete parser;
				parser = NULL;
				return;
			}
			len=fread(buf, 1, sizeof(buf), in);
			if ((pointer==NULL)&&(table>0)&&(table!=12)&&(table<17)){
			    len1=convert2UTF8(buf, buf1, len, table);
			}
			else{
			    memcpy(buf1, buf, len);
			    len1=len;
			}
		} 
		while (!done);

                fclose(in);

		XMLTreeNode * root = parser->RootNode();
		if(!root)
		{	eMessageBox msg(_("XML parse error (rootnode)"), _("Error"), eMessageBox::iconWarning|eMessageBox::btOK);
		        msg.show();     msg.exec();     msg.hide();	
			return;
		}
		if (!strcmp( root->GetType(), "rss")) 
		{	NewsItem thisItem;
			int idcnt = 0;

			for(XMLTreeNode * node = root->GetChild(); node; node = node->GetNext())
			{	if(!strcmp(node->GetType(), "channel"))
				{	for(XMLTreeNode * itemNode = node->GetChild(); itemNode; itemNode = itemNode->GetNext())
					{	if(!strcmp(itemNode->GetType(), "item"))
						{	for(XMLTreeNode * i = itemNode->GetChild(); i; i = i->GetNext())
							{	if(!strcmp(i->GetType(), "title"))
								{	thisItem.title = i->GetData();
									replace_sonderzeichen((char *)thisItem.title.c_str());
									thisItem.title.removeChars('\n');
									thisItem.title.removeChars('\r');
								}
								if(!strcmp(i->GetType(), "description"))
								{	eString desc = i->GetData();
									replace_sonderzeichen((char *)desc.c_str());
									desc.removeChars('\n');
									desc.removeChars('\r');
									desc = removeTags(desc);
									thisItem.description = desc;
								}
							}
							thisItem.id = idcnt++;
							save(thisItem);
						}
					}
				}
			}
		}
		else if(!strcmp(root->GetType(), "rdf:RDF"))
		{	NewsItem thisItem;
			int idcnt = 0;

			for(XMLTreeNode * node = root->GetChild(); node; node = node->GetNext())
			{	if(!strcmp(node->GetType(), "item"))
				{	for(XMLTreeNode * i = node->GetChild(); i; i = i->GetNext())
					{	if(!strcmp(i->GetType(), "title"))
						{	thisItem.title = i->GetData();
							replace_sonderzeichen((char *)thisItem.title.c_str());
							thisItem.title.removeChars('\n');
							thisItem.title.removeChars('\r');
						}
						if(!strcmp(i->GetType(), "description"))
						{	eString desc = i->GetData();
							replace_sonderzeichen((char *)desc.c_str());
//							desc.removeChars('\n');
//							desc.removeChars('\r');
							desc = removeTags(desc);
							thisItem.description = desc;
						}
					}
					thisItem.id = idcnt++;
					save(thisItem);
				}
			}
		}
		else
		{	eString sMsg = eString().sprintf("Unknown file format (%s)", root->GetType());
			eMessageBox msg(sMsg, _("Error"), eMessageBox::iconWarning|eMessageBox::btOK);
			msg.show();     msg.exec();     msg.hide();
		}

		delete parser;
	}
	else
	{	eMessageBox msg(_("RSS file not found"), _("Error"), eMessageBox::iconWarning|eMessageBox::btOK);
		msg.show();     msg.exec();     msg.hide();
	}
}
